/*
 *  ZThreads, a platform-independant, multithreading and 
 *  synchroniation library
 *
 *  Copyright (C) 2001 Eric Crahen, See LGPL.TXT for details
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  SUNY @ Buffalo, hereby disclaims all copyright interest in the
 *  ZThreads library written by Eric Crahen
 */

#ifndef __ZBIASEDRWLOCK_H__
#define __ZBIASEDRWLOCK_H__

#include "zthread/ZThread.h"
#include "zthread/RWLock.h"

namespace ZThread {

/**
 * @class BiasedRWLock
 *
 * @author Eric Crahen
 * @date 03-10-2001
 * @version 1.3.0
 *  
 * A specialized RWLock that has a bias toward writers. It will 
 * prefer writers over readers when many threads are competing for 
 * this lock.
 *
 * @see RWLock 
 */
class ZTHREAD_API BiasedRWLock : RWLock {
  public:
  
  /**
   * Create a new RWLock that has a bias toward writers.
   */
  BiasedRWLock()
    throw(Synchronization_Exception) : RWLock() {}

  /**
   * Destroy the RWLock
   */
  virtual ~BiasedRWLock() throw() {}

  /** 
   * Use the internal counters to allow writers access to the lock
   * before readers, regardless of how many there may be.
   */
  inline virtual bool allowReader() {
     return _activeWriters == 0 && _waitingWriters == 0;
  }
  
};

};

#endif
