/*
 *  ZThreads, a platform-independant, multithreading and 
 *  synchroniation library
 *
 *  Copyright (C) 2001 Eric Crahen, See LGPL.TXT for details
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  SUNY @ Buffalo, hereby disclaims all copyright interest in the
 *  ZThreads library written by Eric Crahen
 */

#ifndef __ZFAIRRWLOCK_H__
#define __ZFAIRRWLOCK_H__

#include "zthread/ZThread.h"
#include "zthread/RWLock.h"

namespace ZThread {

/**
 * @class FairRWLock
 *
 * @author Eric Crahen
 * @date 03-10-2001
 * @version 1.3.0
 *  
 * A specialized RWLock that has a no special bias. It will 
 * maintain a balance between the order readers and writers
 * allowed to acquire this objects locks
 *
 * @see RWLock 
 */
class ZTHREAD_API FairRWLock : RWLock {
  public:
  
  /**
   * Create a fair or somewhat balanced RWLock
   */
  FairRWLock()
    throw(Synchronization_Exception) : RWLock() {}

  /**
   * Destroy this RWLock
   */
  virtual ~FairRWLock() throw() {}

  /** 
   * Use the internal fairness counter to try and keep a balance
   * between the number of readers and the number of writers allowed
   * access to the lock.
   */
  inline virtual bool allowReader() {

    if(_fairnessCounter < 0)
      return _activeWriters == 0 && _waitingWriters == 0;
    else
      return _activeWriters == 0;

  }
  
};

};
#endif
