/*
 *  ZThreads, a platform-independant, multithreading and 
 *  synchroniation library
 *
 *  Copyright (C) 2001 Eric Crahen, See LGPL.TXT for details
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  SUNY @ Buffalo, hereby disclaims all copyright interest in the
 *  ZThreads library written by Eric Crahen
 */

#ifndef __ZNULLMUTEX_H__
#define __ZNULLMUTEX_H__

#include "zthread/ZThread.h"
#include "zthread/Lockable.h"

namespace ZThread { 
  
/**
 * @class NullMutex
 *
 * @author Eric Crahen
 * @version 1.3.1
 * @date 03-18-2001
 *
 * This is an empty Mutex implementation. All the methods return immediately
 * without performing any operation. This is used to perform some types of
 * testing, but has no use in practice.
 */
class ZTHREAD_API NullMutex : public Lockable {
  
  
  // Disable copy constructors
  NullMutex(const NullMutex&);
  NullMutex& operator = (const NullMutex&);
  
  public:

  /**
   * Allocate a new mutex
   *
   * @exception Initialization_Exception - thrown if there are not
   * enough available resources
   */
  inline NullMutex() 
    throw(Synchronization_Exception) {}

  /**
   * Destroy this Mutex object
   */
  inline virtual ~NullMutex()
    throw() {}
  
  /**
   * Null implementation
   * 
   * @exception Synchronization_Exception - never thrown
   */
  inline virtual void acquire() 
    throw(Synchronization_Exception) {}

  /**
   * Null implementation
   * 
   * @param unsigned long - not used
   * @exception Synchronization_Exception - never thrown
   */ 
  inline virtual bool tryAcquire(unsigned long) 
    throw(Synchronization_Exception) { return false; }
  
  /**
   * Null implementation
   * 
   * @exception Synchronization_Exception - never thrown
   */
  inline virtual void release() 
    throw(Synchronization_Exception) {}
  
}; /* NullMutex */


};

#endif
