#ifndef __ZTIMERTHREAD_H__
#define __ZTIMERTHREAD_H__

#include "zthread/ZThread.h"
#include "zthread/Timer.h"

namespace ZThread {

/**
 * @class TimerThread
 *
 * @author Eric Crahen
 * @version 1.5.0
 * @date 07-31-2001
 *
 * This is a Timer object that is based on a Thread. This template
 * can be used to manage the lifetime of the thread correctly. A 
 * static TimerThread should be labeled as a daemon thread using the
 * parameter template. This is very similar to how the Executor
 * templates work.
 *
 * Also, like Executors, static TimerThreads do no need to be joined 
 * while a non-static or local TimerThread should be joined. 
 */
template <bool DAEMON> 
class ZTHREAD_API TimerThread : public Timer, public Thread {
  
  bool _canceled;

 public:

  /**
   * Create a new TimerThread
   */
  TimerThread()
    throw(Synchronization_Exception) {

    if(DAEMON)
      setDaemon(true);

  }

  /**
   * Destroy this TimerThread
   */
  virtual ~TimerThread() throw() { }

  /**
   * Cancel the timer altogether. No further notifications are issued from the
   * timer thread.
   *
   * @exception Synchronization_Exception
   */
  virtual void cancel()
    throw(Synchronization_Exception) {

    _canceled = true;
    interrupt();

  }

  /**
   * Check if the Timer has been Canceled
   *
   * @return bool - cancelation status
   *
   * @exception Synchronization_Exception
   */
  inline virtual bool isCanceled()
    throw(Synchronization_Exception) {

    return _canceled;

  }

  /**
   * Delegate to timer 
   */
  inline virtual void run() throw() {
    Timer::run();
  }

};

}; 

#endif

