#ifndef __ZT_WIN32FASTMUTEX_H__
#define __ZT_WIN32FASTMUTEX_H__

#if !defined(_WIN32)
#error "Win32 specific implementation"
#endif

#include "zthread/ZThread.h"
#include "zthread/Lockable.h"
#include <windows.h>

namespace ZThread {

/**
 * @author Eric Crahen
 * @version 1.5.0
 * @date 07-31-2001
 *
 */ 
class ZTHREAD_API FastMutex : public Lockable {
  
  CRITICAL_SECTION _csMutex;
  DWORD _owner;
  
  public:
  
  /**
   * Create a new FastMutex
   */
  FastMutex() 
    throw(Synchronization_Exception);

  
  /**
   * Destroy FastMutex
   */
  virtual ~FastMutex();

  /**
   * Lock the fast Mutex, no error check.
   * 
   * @exception None
   */ 
  virtual void acquire() 
    throw(Synchronization_Exception);
  
  /**
   * Release the fast Mutex, no error check.
   * 
   * @exception None
   */ 
  virtual void release() 
    throw(Synchronization_Exception);
    
  /**
   * Always returns false. Disabled
   *
   * @return false
   */
  virtual bool tryAcquire(unsigned long)    
    throw(Synchronization_Exception);
  
  
}; /* Fast Mutex */


};
#endif
