/*
 *  ZThreads, a platform-independant, multithreading and 
 *  synchroniation library
 *
 *  Copyright (C) 2001 Eric Crahen, See LGPL.TXT for details
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  SUNY @ Buffalo, hereby disclaims all copyright interest in the
 *  ZThreads library written by Eric Crahen
 */

#ifndef __ZTHREAD_CONFIG_H__
#define __ZTHREAD_CONFIG_H__

// Install a simple debuggin macro
#ifndef NDEBUG

#include <stdio.h>
#define ZTDEBUG printf

#else

inline void _ZDEBUG(...) {
}
#define ZTDEBUG _ZDEBUG

#endif


// Configure the namespace
#ifndef ZTHREAD_NAMESPACE
#define ZTHREAD_NAMESPACE ZThread
#endif

// Clear the API macros to start with
#undef EXPLICIT_TEMPLATE
#undef ZTHREAD_API

#if defined(_WIN32)

#define WIN32_LEAN_AND_MEAN

#pragma warning(disable:4275)
#pragma warning(disable:4290) 
#pragma warning(disable:4786)
#pragma warning(disable:4251)


#endif

// Define the DLL interface for windows shared libraries
#if defined(_WIN32) && !defined(ZTHREAD_STATIC)

#ifdef ZTHREAD_EXPORTS 

#undef ZTHREAD_API
#undef EXPLICIT_TEMPLATE

#define ZTHREAD_API __declspec(dllexport)
#define EXPLICIT_TEMPLATE(X) template class __declspec( dllexport ) X; 

#else

#define ZTHREAD_API __declspec(dllimport)
#define EXPLICIT_TEMPLATE(X) template class __declspec( dllimport ) X; 

#endif // ZTHREAD_EXPORTS 
#else

// Set the API macros to null for POSIX & static builds
#define ZTHREAD_API
#define EXPLICIT_TEMPLATE(X)

#endif

#endif // __ZTHREAD_H__


