// Sleep example to illustrate an interuptable sleep
//
#include <zthread/Guard.h>
#include <zthread/FastMutex.h>
#include <zthread/Thread.h>

#include <iostream>


using namespace ZThread;
using namespace std;

FastMutex mtx;

class LWP : public Runnable {
public:
  virtual ~LWP() throw() {};

  virtual void run() throw() {

    try {

      Guard<FastMutex> guard(mtx);

      cerr << "LWP" << endl;

      Thread::sleep(4000); // gets interrupted

    } catch(Synchronization_Exception& e) { 
      cerr << e.what() << endl;
    } 
    
    cerr << "LWP done" << endl;
  } 

};


class HWP : public Thread {
public:
  virtual ~HWP() throw() {};

  virtual void run() throw() {

    try {

      Guard<FastMutex> guard(mtx);

      cerr << "HWP" << endl;

      Thread::sleep(4000); // gets interrupted

    } catch(Synchronization_Exception& e) { 
      cerr << e.what() << endl;
    }
    
    cerr << "HWP done" << endl;
  } 

};


int main() {

  try {

    LWP lwp;
    Thread t;

    t.run(&lwp);
    t.interrupt();
    t.join();


    HWP hwp;
    hwp.start();
    hwp.interrupt();
    hwp.join();


  } catch(Synchronization_Exception& e) { 
    cerr << e.what() << endl;
  }


#ifdef WIN32
  cerr << "Press a key" << endl;
  getchar(); // Otherwise the console window will close to quickly
#endif

  return 0;

}
