#include <zthread/Guard.h>
#include <zthread/Mutex.h>
#include <zthread/Thread.h>
#include <zthread/ThreadLocal.h>

#define TASKS 5

using namespace ZThread;

#include <iostream>
using namespace std;

ThreadLocal<int> tls;
Mutex mtx;

static int value = 100;

// Lightwieght task
class LWP : public Runnable {
public:

  virtual ~LWP() throw() {};

  virtual void run() throw() {

    Guard<Mutex> guard(mtx);

    cout << "Running...." << endl << endl;
    cout << "Setting TLS" << endl;

    tls.set(value++ );

    cout << "TLS for this thread: " << tls.get() << endl;
    try {

      Thread::current().setPriority(Thread::PRI_NORMAL);
      cout << Thread::current().getPriority() << endl;

    } catch(Synchronization_Exception& e) { 
      cout << e.what() << endl;
      cout << "OK" << endl;
    }

    cout << "OK" << endl;

  }


};


class HWP : public Thread {
public:
  virtual ~HWP() throw() {};

  virtual void run() throw() {

    try {

      Guard<Mutex> guard(mtx);
      tls.set( value++ );

      cout << "TLS for this thread: " << tls.get() << endl;
      
      Thread::sleep(3000); // gets interrupted

    } catch(Synchronization_Exception& e) { 
      cout << e.what() << endl;
      cout << "OK" << endl;
    }
    
    cout << "HWP done" << endl;
  } 

};


int main() {

  try {

    cout << "TLS for main thread: " << tls.get() << endl;

    LWP lwp;
    Thread t[TASKS];

    Thread daemon;
    daemon.setDaemon(true);
    daemon.run(&lwp);

    int n;
    for(n=0;n<TASKS;n++)
      t[n].run(&lwp);


    for(n=0;n<TASKS;n++)
      t[n].join();
    
    
  } catch(Synchronization_Exception& e) { 
    cout << e.what() << endl;
  }


#ifdef WIN32
  cout << "Press a key" << endl;
  getchar(); // Otherwise the console window will close to quickly
#endif

  return 0;

}
